--------------------------------------------------------------------------------
-- 
--------------------------------------------------------------------------------
-- Constants

local CLASS = "CLASSES";
local BLESS	= "BLESSINGS";

local WAR = "WARRIOR";
local LOC = "WARLOCK";
local ROG = "ROGUE";
local PRI = "PRIEST";
local PAL = "PALADIN";
local MAG = "MAGE";
local HUN = "HUNTER";
local DRU = "DRUID";

local MIGHT	= "MIGHT";
local WISD	= "WISDOM";
local KING	= "KINGS";
local SANCT	= "SANCTUARY";
local LIGHT	= "LIGHT";
local SALV	= "SALVATION";

local NORM	= "STANDARD";
local GREAT	= "GREATER";
local NONE	= "NONE";

local OPT	= "OPTIONS";
local SAVE	= "Save";
local LOAD	= "Load";
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Variables
local GPTargets = {};
local GPNextClass = nil;

local GPNames =
{ 
	[BLESS] = 
	{
		[MIGHT] =
		{
			[GREAT] = "Greater Blessing of Might",
			[NORM] = "Blessing of Might"
		},
		[WISD] =
		{
			[GREAT] = "Greater Blessing of Wisdom",
			[NORM] = "Blessing of Wisdom"
		},
		[KING] =
		{
			[GREAT] = "Greater Blessing of Kings",
			[NORM] = "Blessing of Kings" 
		},
		[SANCT] =
		{
			[GREAT] = "Greater Blessing of Sanctuary",
			[NORM] = "Blessing of Sanctuary"
		},
		[LIGHT] =
		{
			[GREAT] = "Greater Blessing of Light",
			[NORM] = "Blessing of Light",
		},
		[SALV] =
		{
			[GREAT] = "Greater Blessing of Salvation",
			[NORM] = "Blessing of Salvation"
		},
		[NONE] = "None"
	},
	[CLASS] = 
	{
		[WAR]	= "Warrior",
		[LOC]	= "Warlock",
		[ROG]	= "Rogue",
		[PRI]	= "Priest",
		[PAL]	= "Paladin",
		[MAG]	= "Mage",
		[HUN]	= "Hunter",
		[DRU]	= "Druid"
	},
	[OPT] = 
	{
		[SAVE] = "Save",
		[LOAD] = "Load"
	}
};
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--[[
  *	Function:  GPFind()
  *	
  *	Description:  Walks through party and raid lists to find one target
  *		      for each class to use in buffing.  Only classes present
  *		      will get set to be buffed.
  *		      
  *		      By yourself, it will buff whatever you have PALADIN's set
  *		      In a Party, it will buff every class present in the party
  *		      In a Raid, it will buff every class present in the raid
  *]]
local function GPFind()
	local NewTargets =
	{
		[WAR] = nil,
		[LOC] = nil,
		[ROG] = nil,
		[PRI] = nil,
		[PAL] = nil,
		[MAG] = nil,
		[HUN] = nil,
		[DRU] = nil
	};
	
	-- Set ourself as the PALADIN target
	NewTargets[PAL] = UnitName("player");
	
	-- Determine group settings
	local InRaid, InParty = false, false;
	if (UnitInRaid("player")) then
		InRaid = true;
	elseif (UnitInParty("player")) then
		InParty = true;
	end
	
	-- Used to quickly determine when we have filled the target list
	local TargetsRemaining = 7;
	
	-- Find targets in raid
	if (InRaid) then
		for i=1,40 do
			x,class = UnitClass("raid"..i);
			if (UnitExists("raid"..i) and UnitIsVisible("raid"..i) and (not NewTargets[class])) then
				NewTargets[class] = UnitName("raid"..i);
				TargetsRemaining = TargetsRemaining - 1;
			end
			if (TargetsRemaining < 1) then i = 40; end
		end
	-- Find targets in party
	elseif (InParty) then
		for i=1,4 do
			x,class = UnitClass("party"..i);
			if (UnitExists("party"..i) and UnitIsVisible("party"..i) and (not NewTargets[class])) then
				NewTargets[class] = UnitName("party"..i);
				TargetsRemaining = TargetsRemaining - 1;
			end
			if (TargetsRemaining < 1) then i = 4; end
		end
	end

	-- Set new targets
	GPTargets = NewTargets;
end
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--[[
  *	Function:  GPTranslate(key)
  *	
  *	Description:  Return the appropriate spell or class name based on the KEY given
  *
  *	Parameters:
  *	key:	The keyword to lookup
  *]]
local function GPTranslate(key)
	key = string.upper(key);
	
	-- Check blessings
	if ((key == "MIGHT")) then return MIGHT;
	elseif ((key == "WISDOM")or(key == "WIS")) then return WISD;
	elseif ((key == "KINGS")or(key == "KING")) then return KING;
	elseif ((key == "SANCTUARY")or(key == "SANCT")) then return SANCT;
	elseif ((key == "LIGHT")) then return LIGHT;
	elseif ((key == "SALVATION")or(key == "SALV")) then return SALV;
	elseif (key == "NONE") then return NONE;
	-- Check class
	elseif (key == string.upper(GPNames[CLASS][WAR])) then return WAR;
	elseif (key == string.upper(GPNames[CLASS][LOC])) then return LOC;
	elseif (key == string.upper(GPNames[CLASS][ROG])) then return ROG;
	elseif (key == string.upper(GPNames[CLASS][PAL])) then return PAL;
	elseif (key == string.upper(GPNames[CLASS][PRI])) then return PRI;
	elseif (key == string.upper(GPNames[CLASS][MAG])) then return MAG;
	elseif (key == string.upper(GPNames[CLASS][HUN])) then return HUN;
	elseif (key == string.upper(GPNames[CLASS][DRU])) then return DRU;
	else return nil; end
end
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--[[
  *	Function:  GPSetBuff(class, type)
  *	
  *	Description:  For the given class, associate the given Blessing. The
  *				  strings are case insensitive
  *
  *	Parameters:
  *	class:	The class-name this blessing will be associated with
  *	type:	The "X" in "Greater Blessing of X"
  *]]
function GPSetBuff(class, type)
	-- Get correct class
	class = GPTranslate(string.upper(class));
	
	-- Get correct blessing
	type = GPTranslate(type);
	
	if (not type) then
		DEFAULT_CHAT_FRAME:AddMessage("    |cffffff00GreaterPally:|r Invalid Blessing");
		return;
	elseif (not class) then
		DEFAULT_CHAT_FRAME:AddMessage("    |cffffff00GreaterPally:|r Invalid Class");
		return;
	end
	
	-- Set the buff, and reset the buff sequence
	--local glyph = getglobal("GPFrame"..class.."_Drop");
	--GreaterPally_OnBtnClick(glyph, type);
	GPNextClass = nil;

	DEFAULT_CHAT_FRAME:AddMessage("    |cffffff00GreaterPally:|r "..class.." buff set to |cff00ff00"..GreaterPally_Buffs[class].."|r");
end
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--[[
  *	Function:  GreaterPally()
  *	
  *	Description:  Actual buffing function.  Requires you to use 
  *		      GPSetBuff(c,t) for each class you want to be able to buff,
  *		      whether they are actually present or not (otherwise GP has
  *		      no way to know what you would want to buff them with).
  *		      
  *		      Due to Blizzard's limitations, you can only cast 1 spell
  *		      per keypress.  So to buff all the classes, keep using this
  *		      function until you get the message "End of buffing".
  *		      At the start of each sequence, targets are selected and set.
  *]]
function GreaterPally()

	-- Watch for global cooldown, so we dont "cast" when we can't
	local spellID = 1;
	while true do
		local spellName = GetSpellName(spellID, BOOKTYPE_SPELL)
		
		if (not spellName) then break; end
		if (string.find(spellName, "Blessing")) then break; end
		spellID = spellID + 1;
	end
	if (GetSpellCooldown(spellID, BOOKTYPE_SPELL) ~= 0) then return; end
	
	-- If we are starting a new buff-sequence, find available targets
	if (GPNextClass == nil) then
		GPFind();
	end
	
	-- Get class to buff
	GPNextClass = next(GPTargets, GPNextClass);
	
	-- Done buffing!
	if (GPNextClass == nil) then
		DEFAULT_CHAT_FRAME:AddMessage("    |cffffff00GreaterPally: End of buff sequence reached.|r");
		return;
	end
	
	-- Check if we...
	--		a) have a target for this class 
	--		b) have a blessing set for this class
	local blessing = GreaterPally_Buffs[GPNextClass];
	local target = GPTargets[GPNextClass];
	
	-- If no blessing set, or set to none, or no target, skip to next class
	if ((blessing == nil) or (blessing == NONE) or (target == nil)) then
		return GreaterPally();
	end

	-- Otherwise, buff away!
	-- Use Greater blessings for raids, 5min otherwise.
	-- Eventually I'll make it an option you choose.
	local spell = nil;

	if (UnitInRaid("player")) then
		spell = GPNames[BLESS][blessing][GREAT];
	else
		spell = GPNames[BLESS][blessing][NORM];
	end

	if (GPNextClass and target and spell) then
		DEFAULT_CHAT_FRAME:AddMessage("    |cffffff00GreaterPally:|r Buffing "..GPNextClass.."'s with |cff00ff00"..spell.."|r");
		TargetByName(target);
		CastSpellByName(spell);
		TargetLastTarget();
	else
		DEFAULT_CHAT_FRAME:AddMessage("    |cffffff00GreaterPally:|r <Unable to buff>");
	end
end
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--[[
  *	Function:  GP_OnMsg(msg)
  *	
  *	Description:  Slash-handler
  *]]
function GP_OnMsg(msg)
	msg = string.upper(msg);
	
	if ((msg == "") or (msg == nil) or (msg == "TOGGLE")) then
		local glyph = getglobal("GPFrame");
		if (glyph:IsVisible()) then
			glyph:Hide();
		else
			glyph:Show();
		end
	elseif (msg == "SHOW") then
		getglobal("GPFrame"):Show();
	elseif (msg == "HIDE") then
		getglobal("GPFrame"):Hide();
	elseif (msg == "BUFFS") then
		for class,spell in GreaterPally_Buffs do
			if (spell and (class ~= "Init")) then
				DEFAULT_CHAT_FRAME:AddMessage("    |cffffff00GreaterPally:|r "..class.." buff set to |cff00ff00"..spell.."|r");
			end
		end
	end
end
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--[[
  *	Function:  GP_OnLoad()
  *	
  *	Description:  Lets you know GreaterPally has loaded.
  *]]
function GP_OnLoad()
	DEFAULT_CHAT_FRAME:AddMessage("GreaterPally Loaded");
	this:RegisterEvent("VARIABLES_LOADED");
	
	SLASH_GREATERPALLY1 = "/greaterpally";
	SLASH_GREATERPALLY2 = "/gp";
	SlashCmdList["GREATERPALLY"] = function(msg)
		GP_OnMsg(msg);
	end
end
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--[[
  *	Function:  GP_Event()
  *	
  *	Description: 
  *]]
function GP_OnEvent(...)
  if (arg[1] == "VARIABLES_LOADED") then
  
  	if (GreaterPally_Buffs == nil) then
		GreaterPally_Buffs = {
			[WAR]	= nil,
			[LOC]	= nil,
			[ROG]	= nil,
			[PRI]	= nil,
			[PAL]	= nil,
			[MAG]	= nil,
			[HUN]	= nil,
			[DRU]	= nil,
		};
	end
	
	if (GreaterPally_Sets == nil) then
		GreaterPally_Sets = {};
	end

	local glyph = nil;
	for key,val in pairs(GreaterPally_Buffs) do
		if (key ~= "Init") then
			glyph = getglobal("GPFrame"..key.."_Drop");
			DEFAULT_CHAT_FRAME:AddMessage(glyph:GetName());
			UIDropDownMenu_SetSelectedID(glyph, glyph:GetID());
			UIDropDownMenu_SetSelectedValue(glyph, val);
		end
	end
	
	glyph = getglobal("GPFrameLoad_Drop");
	local funct = function()
			local info;
			for set,key in pairs(GreaterPally_Sets) do
				info = {};
				info.text = set;
				info.func = GreaterPally_OnBtnClick;
				info.arg1 = glyph;
				info.arg2 = set;
				UIDropDownMenu_AddButton(info);
			end
		end
	UIDropDownMenu_Initialize(glyph, funct);
	
	this:UnregisterEvent("VARIABLES_LOADED");
  end
end
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--[[
  *	Function:  GreaterPally_OnBtnClick(glyph, sel)
  *	
  *	Description: Called when a button-type object is clicked in the GUI
  *
  * Parameters:
  * Glpyh:  The object that generated the event
  * sel:    Extra data as needed (really handy for the Dropdown menu's)
  *]]
function GreaterPally_OnBtnClick(glyph, sel)
	local name = glyph:GetName();
	local index;
	
	if (string.find(name, "Load_Drop")) then
		UIDropDownMenu_SetSelectedID(glyph, glyph:GetID());
		UIDropDownMenu_SetSelectedValue(glyph, sel);
		
		currentSet = sel;
	elseif (string.find(name, "_Drop")) then
		index = string.gsub(name, (glyph:GetParent()):GetName(), "", 1);
		index = string.gsub(index, "_Drop", "", 1);
		
		UIDropDownMenu_SetSelectedID(glyph, glyph:GetID());
		UIDropDownMenu_SetSelectedValue(glyph, sel);
		
		GreaterPally_Buffs[index] = sel;
	elseif (string.find(name, "_Button")) then
		index = string.gsub(name, (glyph:GetParent()):GetName(), "", 1);
		index = string.gsub(index, "_Button", "", 1);
		
		if (index == "Load") then
			local set = GreaterPally_Sets[currentSet];
			local bless;
			for class,spell in pairs(set) do			
				if (spell and (class ~= "Init")) then
					bless = set[class];
					GreaterPally_Buffs[class] = bless;
					
					local drop = getglobal("GPFrame"..class.."_Drop");
					UIDropDownMenu_SetSelectedID(drop, drop:GetID());
					UIDropDownMenu_SetSelectedValue(drop, bless);
				end
			end
		elseif (index == "Save") then
			index = getglobal("GPFrameSave_Edit"):GetText();
			GreaterPally_Sets[index] = {};
			for key,val in pairs(GreaterPally_Buffs) do
				GreaterPally_Sets[index][key] = val;
			end
		end
	end
end
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--[[
  *	Function:  GreaterPally_Initialize(glyph)
  *	
  *	Description: Initializes objects in the GUI
  *
  * Parameters:
  * Glpyh:  The object to initialize
  *]]
function GreaterPally_Initialize(glyph)
	local name = glyph:GetName();
	local index;

	if (string.find(name, "Load_Drop")) then
	elseif (string.find(name, "_Drop")) then
	
		index = string.gsub(name, (glyph:GetParent()):GetName(), "", 1);
		index = string.gsub(index, "_Drop", "", 1);
	
		local funct = function()
			local info;
			for key in pairs(GPNames[BLESS]) do
				info = {};
				info.text = key;
				info.func = GreaterPally_OnBtnClick;
				info.arg1 = glyph;
				info.arg2 = key;
				UIDropDownMenu_AddButton(info);
			end
		end
		UIDropDownMenu_Initialize(glyph, funct);
		
	elseif (string.find(name, "_Static")) then
		index = string.gsub(name, (glyph:GetParent()):GetName(), "", 1);
		index = string.gsub(index, "_Static", "", 1);
		getglobal(name.."_Text"):SetText(GPNames[CLASS][index]);
	elseif (string.find(name, "_Button")) then
		index = string.gsub(name, (glyph:GetParent()):GetName(), "", 1);
		index = string.gsub(index, "_Button", "", 1);
		glyph:SetText(GPNames[OPT][index]);
	end
end